"use strict";
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
    if (kind === "m") throw new TypeError("Private method is not writable");
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
};
var _Files_maxFileSize, _Files_size, _Files_stream, _Files_lock, _Files_debounce, _Files_initialized, _Files_queue, _Files_queueIdx;
Object.defineProperty(exports, "__esModule", { value: true });
exports.Files = exports.FileAdapter = void 0;
const utils_1 = require("@hibas123/utils");
const fs = require("fs");
const path = require("path");
const logging_1 = require("@hibas123/logging");
const events_1 = require("events");
const MAX_FILE_SIZE = 500000000;
class FileAdapter {
    constructor(filename, maxFileSize = MAX_FILE_SIZE) {
        this.filename = filename;
        this.maxFileSize = maxFileSize;
        this.level = logging_1.LoggingTypes.Debug;
        this.isInit = new utils_1.AwaitStore(false);
    }
    setLevel(level) {
        this.level = level;
    }
    async init() {
        if (!this.file) {
            this.file = Files.getFile(this.filename);
            await this.file
                .init(this.maxFileSize)
                .then(() => this.isInit.send(true));
        }
    }
    flush(sync) {
        // return this.file.flush(sync);
    }
    onMessage(message) {
        let msg = Buffer.from(logging_1.Formatted.strip(message.text) + "\n");
        this.file.write(msg);
    }
    async close() {
        if (this.file) {
            await this.file.close();
            this.file = undefined;
        }
        this.isInit.send(false);
    }
}
exports.FileAdapter = FileAdapter;
//TODO: Optimise write path
const Debounce = (callback, iv = 500, max = 100) => {
    let to;
    let curr = 0;
    return {
        trigger: () => {
            curr++;
            if (curr >= max) {
                curr = 0; // not clearing timeout, since this is a very high cost operation
                callback();
            }
            else if (!to) {
                to = setTimeout(() => {
                    to = undefined;
                    curr = 0;
                    callback();
                }, iv);
            }
        },
    };
};
const QUEUE_START_SIZE = 10000;
class Files {
    static getFile(filename) {
        filename = path.resolve(filename);
        let file = this.files.get(filename);
        if (!file) {
            file = new Files(filename);
            this.files.set(filename, file);
        }
        file.open++;
        return file;
    }
    get initlialized() {
        return __classPrivateFieldGet(this, _Files_initialized, "f");
    }
    constructor(file) {
        this.file = file;
        this.open = 0;
        _Files_maxFileSize.set(this, MAX_FILE_SIZE);
        _Files_size.set(this, 0);
        _Files_stream.set(this, undefined);
        _Files_lock.set(this, new utils_1.Lock());
        _Files_debounce.set(this, Debounce(this.checkQueue.bind(this)));
        _Files_initialized.set(this, false);
        _Files_queue.set(this, new Array(QUEUE_START_SIZE));
        _Files_queueIdx.set(this, 0);
    }
    async init(maxFileSize) {
        if (__classPrivateFieldGet(this, _Files_initialized, "f"))
            return;
        __classPrivateFieldSet(this, _Files_maxFileSize, maxFileSize, "f");
        let lock = await __classPrivateFieldGet(this, _Files_lock, "f").getLock();
        const folder = path.dirname(this.file);
        if (folder) {
            if (!(await fsExists(folder))) {
                await fsMkDir(folder).catch(() => { }); //Could happen, if two seperate instances want to create the same folder so ignoring
            }
        }
        await this.initializeFile();
        __classPrivateFieldSet(this, _Files_initialized, true, "f");
        await this.checkQueue(true);
        lock.release();
    }
    async initializeFile(new_file = false) {
        try {
            if (__classPrivateFieldGet(this, _Files_stream, "f")) {
                const closePrms = (0, events_1.once)(__classPrivateFieldGet(this, _Files_stream, "f"), "close");
                __classPrivateFieldGet(this, _Files_stream, "f").end();
                await closePrms;
            }
            let size = 0;
            if (await fsExists(this.file)) {
                let stats = await fsStat(this.file);
                if (new_file || stats.size >= __classPrivateFieldGet(this, _Files_maxFileSize, "f")) {
                    if (await fsExists(this.file + ".old"))
                        await fsUnlink(this.file + ".old");
                    await fsMove(this.file, this.file + ".old");
                }
                else {
                    size = stats.size;
                }
            }
            __classPrivateFieldSet(this, _Files_stream, fs.createWriteStream(this.file, { flags: "a" }), "f");
            __classPrivateFieldSet(this, _Files_size, size, "f");
        }
        catch (err) {
            console.log(err);
            //TODO: is this the right behavior? Probably not...
            process.exit(1);
        }
    }
    async checkQueue(nolock = false) {
        let lock;
        if (nolock == false) {
            //TODO: New design might cause new messages to be "stalled" till close or another message
            if (__classPrivateFieldGet(this, _Files_lock, "f").locked)
                return;
            lock = await __classPrivateFieldGet(this, _Files_lock, "f").getLock();
        }
        const queue = __classPrivateFieldGet(this, _Files_queue, "f");
        const queueCnt = __classPrivateFieldGet(this, _Files_queueIdx, "f");
        __classPrivateFieldSet(this, _Files_queue, new Array(QUEUE_START_SIZE), "f");
        __classPrivateFieldSet(this, _Files_queueIdx, 0, "f");
        let buffer = Buffer.alloc(1024 * 128);
        let ci = 0;
        for (let i = 0; i < queueCnt; i++) {
            const entry = queue[i];
            if (entry.length + ci > buffer.length) {
                await this.write_to_file(buffer.slice(0, ci));
                ci = 0;
                if (entry.length > buffer.length) {
                    await this.write_to_file(entry);
                }
                else {
                    entry.copy(buffer, ci);
                    ci += entry.length;
                }
            }
            else {
                entry.copy(buffer, ci);
                ci += entry.length;
            }
        }
        if (ci > 0) {
            await this.write_to_file(buffer.slice(0, ci));
        }
        if (lock)
            lock.release();
    }
    async close() {
        //TODO: maybe some raceconditions when open collides with close
        const lock = await __classPrivateFieldGet(this, _Files_lock, "f").getLock();
        await this.checkQueue(true);
        this.open--;
        if (this.open <= 0) {
            const a = (0, events_1.once)(__classPrivateFieldGet(this, _Files_stream, "f"), "close");
            __classPrivateFieldGet(this, _Files_stream, "f").close();
            await a;
            Files.files.delete(this.file);
        }
        lock.release();
    }
    async write_to_file(data) {
        try {
            if (data.byteLength < __classPrivateFieldGet(this, _Files_maxFileSize, "f") &&
                __classPrivateFieldGet(this, _Files_size, "f") + data.byteLength > __classPrivateFieldGet(this, _Files_maxFileSize, "f")) {
                await this.initializeFile(true);
            }
            __classPrivateFieldSet(this, _Files_size, __classPrivateFieldGet(this, _Files_size, "f") + data.byteLength, "f");
            __classPrivateFieldGet(this, _Files_stream, "f").write(data);
        }
        catch (err) {
            // TODO: Better error handling!
            console.error(err);
            this.initializeFile(false);
            this.write_to_file(data);
        }
    }
    write(data) {
        var _a, _b;
        __classPrivateFieldGet(this, _Files_queue, "f")[__classPrivateFieldSet(this, _Files_queueIdx, (_b = __classPrivateFieldGet(this, _Files_queueIdx, "f"), _a = _b++, _b), "f"), _a] = data;
        __classPrivateFieldGet(this, _Files_debounce, "f").trigger();
    }
}
exports.Files = Files;
_Files_maxFileSize = new WeakMap(), _Files_size = new WeakMap(), _Files_stream = new WeakMap(), _Files_lock = new WeakMap(), _Files_debounce = new WeakMap(), _Files_initialized = new WeakMap(), _Files_queue = new WeakMap(), _Files_queueIdx = new WeakMap();
Files.files = new Map();
function fsUnlink(path) {
    var _a;
    if ((_a = fs.promises) === null || _a === void 0 ? void 0 : _a.unlink) {
        return fs.promises.unlink(path);
    }
    return new Promise((resolve, reject) => {
        fs.unlink(path, (err) => {
            if (err)
                reject(err);
            else
                resolve();
        });
    });
}
function fsStat(path) {
    var _a;
    if ((_a = fs.promises) === null || _a === void 0 ? void 0 : _a.stat) {
        return fs.promises.stat(path);
    }
    return new Promise((resolve, reject) => {
        fs.stat(path, (err, stats) => {
            if (err)
                reject(err);
            else
                resolve(stats);
        });
    });
}
function fsMove(oldPath, newPath) {
    return new Promise((resolve, reject) => {
        let callback = (err) => {
            if (err)
                reject(err);
            else
                resolve();
        };
        fs.rename(oldPath, newPath, function (err) {
            if (err) {
                if (err.code === "EXDEV") {
                    copy();
                }
                else {
                    callback(err);
                }
                return;
            }
            callback();
        });
        function copy() {
            fs.copyFile(oldPath, newPath, (err) => {
                if (err)
                    callback(err);
                else
                    fs.unlink(oldPath, callback);
            });
        }
    });
}
function fsExists(path) {
    return new Promise((resolve, reject) => {
        fs.access(path, (err) => resolve(!err));
    });
}
function fsMkDir(path) {
    return new Promise((resolve, reject) => {
        fs.mkdir(path, (err) => (err ? reject(err) : resolve()));
    });
}
//# sourceMappingURL=filewriter.js.map